// programme de Frederic LETELLIER-COHEN

#include "quadrex200.h"

#include "send_receive.cpp" // Fonctions pour envoyer et recevoir
#include "fonctions_dialogues.cpp" // Fonctions de dialogue avec le TPG300

QUADREX200::QUADREX200(QWidget *parent) : QMainWindow(parent)
{
  ui.setupUi(this); // chargement de l'interface graphique
  port = NULL; // initialisation du port serie
}


///////////////////////////////////////////
// donne la date et l'heure de la mesure
void QUADREX200::donneheure(void)
{
  QDateTime date = QDateTime::currentDateTime();
  date_init = date;
  date_du_jour = date.toString( "dd/MM/yyyy");

  ui.lineEdit_1_1->setText(date.toString( "dd/MM/yyyy") ); // Date de dpart
  ui.lineEdit_1_2->setText(date.toString( "HH:mm:ss") ); // Heure de dpart
}


///////////////////////////////////////////
// donne le nom du fichier
void QUADREX200::donnenomfichier(void)
{
  QString date = date_init.toString( "yyyy-MM-dd-HH-mm-ss");
  QString nom_file = "data/data_QUADREX200_" + date + ".csv";
  nom_fichier = nom_file;
  ui.textEdit_1_1->setText( nom_fichier ); // Nom du fichier d'enregistrement
}

///////////////////////////////////////////
// Connexion a l'appareil
int QUADREX200::connexion(void)
{

//1 bit de dmarrage = "0"
//7 bits de donnes = valeur comprise entre 0 et 127
//1 bit de parit pair
//1 bit de stop = "1"
//avec un controle de flux materiel

  QFile file;
  file.setFileName("config/config_rs232.ini");
  if( !file.exists() )
    return -1;      

  file.open(QIODevice::ReadOnly | QIODevice::Text );
  QTextStream in(&file);
  QString nomport = in.readLine(); 
  file.close();

  PortSettings settings; // COM1 ou ttyS0
  settings.BaudRate = BAUD9600;    // 9600 bauds
  settings.Parity = PAR_EVEN;    // pas de parite
  settings.DataBits = DATA_7;   // 7 bits de donnees
  settings.StopBits = STOP_1;    // bit de stop 1.0
  settings.FlowControl = FLOW_HARDWARE;    // bit de stop 1.0
  settings.Timeout_Millisec = 500;    // delai de timeout fixe a 500ms
 
  // si la connexion existe deja, on la ferme
  if(port != NULL)
  {
	   port->close(); // on ferme le port
       delete port;
       port = NULL;
  }

  // connexion
  port = new QextSerialPort(nomport,settings); 

  // ouverture du port
  port->open(QIODevice::ReadWrite); // on ouvre le port

  // on regarde si le port est bien ouvert
  if(port->isOpen() != 1)
    return -1; // erreur

   // on remet tous les parametres pour etre sur qu'ils ont bien ete pris en compte
   port->setBaudRate(BAUD9600);    // 9600 bauds
   port->setDataBits(DATA_7);   // 7 bits de donnees
   port->setParity(PAR_EVEN );    // parite pair
   port->setFlowControl(FLOW_HARDWARE); // controle materiel
   port->setStopBits(STOP_1);    // bit de stop 1
   int retour = (int)port;
   return retour;
} 

///////////////////////////////////////////
// deConnexion a l'appareil
void QUADREX200::deconnexion(void)
{
  // si la connexion existe deja, on la ferme
  if(port != NULL)
  {
	   port->close(); // on ferme le port
       delete port;
       port = NULL;
  }
} 

/////////////////////////////////////////////
// Configuration de la connexion RS232
/////////////////////////////////////////////
void QUADREX200::on_pushButton_1_1_clicked()
{
  // on affiche la fenetre
  fenetre_rs232.show();
}

/////////////////////////////////////////////
// Configuration de PARAMETRES
/////////////////////////////////////////////
void QUADREX200::on_pushButton_1_2_clicked()
{
  // on affiche la fenetre
  fenetre_parametres.show();
}

/////////////////////////////////////////////
// Enregistrement des parametres
/////////////////////////////////////////////
void QUADREX200::on_pushButton_1_3_clicked()
{
  donneheure(); // affichage de l'heure
  donnenomfichier(); // donne le nom du fichier

  ui.lineEdit_1_3->setText("Enregistrement en cours..."); // descriptif de l'erreur
  ui.lineEdit_1_3->setStyleSheet("background: white");


  // connexion a l'appareil
  int erreur;
  erreur = connexion(); // permet de se connecter a l'appareil
  if(erreur == -1)
  {
    ui.lineEdit_1_3->setText("Erreur ouverture de port serie"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }

  // recuperation des bornes
  int borne_min;
  int borne_max;
  borne_min = fenetre_parametres.communique_valeur1();
  borne_max = fenetre_parametres.communique_valeur2();
  if(borne_min >= borne_max)
  {
    ui.lineEdit_1_3->setText("Erreur sur les bornes de mesure"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }

  // ouverture du fichier de donnees
  QFile file;
  file.setFileName(nom_fichier);
  if (!file.open(QIODevice::Append | QIODevice::Text ))
  {
    // il y a un probleme pour ouvrir le fichier
    ui.lineEdit_1_3->setText("Erreur d'ouverture du fichier"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;        
  }
  QTextStream out(&file);

  // enregistrement de la date
  out << "Enregistrement ralis le : " << date_init.toString( "dd/MM/yyyy  HH:mm:ss") << endl << endl;

  // travail en mode reciproque
  QString reciproque2 = reciproque();
  out << "On travaille en mode rciproque (rcupration des donnes prsentes en mmoire)" << endl;
  out << "commande envoye : <STX</A/<TRM>" << endl << endl;
  if(reciproque2 != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur le mode reciproque"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }

  // indicateur du mode reciproque - analogique
  QString indicateurreciproque2 = indicateurreciproque();
  out << "On slectionne les donnees stockes en mode analogique" << endl;
  out << "commande envoye : <STX>/D/1/<TRM>" << endl << endl;
  if(indicateurreciproque2 != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur le mode reciproque analogique"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }

  // delimitation des bornes
  QString delimitationindex = index(borne_min,borne_max);
  out << "On dlimite les valeurs de l'index" << QString::number(borne_min) << endl;
  out << "commande envoye : <STX>/I/0/ /200/<TRM>" << endl << endl;
  if(delimitationindex != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur les bornes de la mesure"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }


  // recuperation du gain
  QString gain = choixparametre("0");
  if(gain != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur le parametre gain"); // descriptif de l'erreur
    return;
  }
  out << "On slectionne le paramtre gain de l'affichage et on demande sa lecture" <<endl;
  out << "commandes envoyes : <STX>/P/0/<TRM> et <STX>/F/<TRM>" << endl;
  gain = lectureparametre();
  out << "valeur brute du gain de l'affichage est : " << gain <<endl;
  int gainintgauche, gainintdroite;
  QString gaingauche, gaindroite;
  gaingauche = gain.left(2);
  gaindroite = gain.right(2);
  gainintgauche = gaingauche.toInt();
  gainintdroite = gaindroite.toInt();
  gain = QString::number(gainintgauche) + "E-" + QString::number(gainintdroite);
  out << "La valeur du gain est de : " << gain <<endl <<endl;

  // recuperation du balayage
  QString balayage = choixparametre("1");
  if(balayage != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur le parametre balayage"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }
  out << "On slectionne le paramtre balayage de l'affichage et on demande sa lecture" <<endl;
  out << "commandes envoyes : <STX>/P/1/<TRM> et <STX>/F/<TRM>" << endl;
  balayage = lectureparametre();
  out << "La valeur brute renvoye par l'appareil est : " << balayage << endl;
  int balayageint = balayage.toInt();
  if(balayageint == 0)
    balayage = "trs lente (0)"; 
  else if(balayageint == 1)
    balayage = "lente (1)";
  else if(balayageint == 2)
    balayage = "normale (2)";
  else if(balayageint == 3)
    balayage = "rapide (3)";
  else if(balayageint == 4)
    balayage = "trs rapide (4)";
  out << "La vitesse de balayage de l'affichage est : " << balayage << endl << endl;


  // recuperation du centre de masse
  QString centremasse = choixparametre("3");
  if(centremasse != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur le parametre centre de masse"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: red");
    return;
  }
  out << "On slectionne le paramtre masse centrale et on demande sa lecture" <<endl;
  out << "commandes envoyes : <STX>/P/3/<TRM> et <STX>/F/<TRM>" << endl;
  centremasse = lectureparametre();
  out << "La valeur brute renvoye par l'appareil est : " << centremasse << endl;
  int centremasseint = centremasse.toInt();
  centremasse=QString::number(centremasseint);
  out << "La valeur de la masse centrale est : " << centremasse << endl << endl;


/*
  // recuperation de la largeur
  QString largeur = choixparametre("4");
  if(largeur != "ok")
  {
    ui.lineEdit_1_3->setText("Erreur sur le parametre largeur"); // descriptif de l'erreur
    return;
  }
  out << "On slectionne le paramtre largeur et on demande sa lecture" <<endl;
  out << "commandes envoyes : <STX>/P/4/<TRM> et <STX>/F/<TRM>" << endl;
  largeur = lectureparametre();
  out << "La valeur brute renvoye par l'appareil est : " << largeur << endl;
  out << endl;
*/

  // recuperation des mesures
  QString toutes_les_mesures = getvaleurs();
  out << "Rcupration des valeurs" << endl;
  file.close();


  traitementdonnnees(toutes_les_mesures, centremasseint, gain);
  ui.lineEdit_1_3->setText("Rception finie et sans erreur"); // descriptif de l'erreur
    ui.lineEdit_1_3->setStyleSheet("background: green");


  // fermeture du fichier et deconnexion de l'appareil
  file.close();
  deconnexion(); // permet de se deconnecter a l'appareil
}



/////////////////////////////////////////////
// Traitement des donnees
/////////////////////////////////////////////
void QUADREX200::traitementdonnnees(QString texte, int massecentrale, QString gain)
{
  QFile file;
  file.setFileName(nom_fichier);
  file.open(QIODevice::Append | QIODevice::Text );
  QTextStream out(&file);

  int compteur = 0;
  double mass = massecentrale - 10;
  QString massstring;
  QString valeuramplitude;
  double valeuramplitudedouble;

//  out << "INDEX" << "\x09" << "MASS" << "\x09" << "AMPLITUDE" << "\x09" << "GAIN" << endl;
  out << "INDEX" << ";" << "MASS" << ";" << "AMPLITUDE" << ";" << "GAIN" << endl;

  while(texte.size() >= 3)
  {
//    out << QString::number(compteur) << "\x09";
    out << QString::number(compteur) << ";";
    compteur++;

    massstring = QString::number(mass);
    massstring = massstring.replace(".", ",");
//    out << massstring << "\x09";
    out << massstring << ";";
    mass = mass + 0.1;
    
    valeuramplitude = texte.left(3);
    valeuramplitude.insert(2, QString("."));
    valeuramplitudedouble = valeuramplitude.toDouble();
    valeuramplitude = QString::number(valeuramplitudedouble);
    valeuramplitude = valeuramplitude.replace(".", ",");
//    out << valeuramplitude << "\x09"; // on enregistre les 3 premiers caracteres
    out << valeuramplitude << ";"; // on enregistre les 3 premiers caracteres
    texte.remove(0,3); // on retire les 3 premiers caracteres

    out << gain << endl; // on enregistre la valeur du gain

  }

  file.close();

  return;
}

